#
# This is a Shiny web application. You can run the application by clicking
# the 'Run App' button above.
#
# Find out more about building applications with Shiny here:
#
#    http://shiny.rstudio.com/
#


##
##Using ggplot and plotly
##  ggplot is  fairly easy to integrate within shiny--there is just one thing to remember.
## we just need to create a plotOutput in the UI,and a renderPlot in the server, and put the 
## ggplot code in the server function. However, in order for it to work, you need to assign 
## your ggplot object a name, and then print the name, like this:
##
##  p <- ggplot(data,aes(x=x))+geom_point()
##  print(p)
##
##
##  Plotly  is an interactive graphing function that makes nice graphics akin to ggplot, but for posting online.
## It requires some alternative rendering and output functions. We must use plotlyOutput and renderPlotly instead
##
##  Plotly plots don't use ggplot grammar, but it should be easy to create a simple graph (maybe in the console first).
##
##
## Instructions:
## 1. create a ggplot graph, possibly plotting the data() value against some other randomly generated value. 
## 2. create a plotly graph showing something similar.

## Be sure these are installed!
library(ggplot2)
library(plotly)
library(shiny)

# Define UI for application that draws a histogram
ui <- fluidPage(
  
  
  # Application title
  titlePanel("Random distribution generation"),
  
  # Sidebar with a slider input for number of bins 
  sidebarLayout(
    sidebarPanel(
      sliderInput("bins",                        "Number of bins:",
                  min = 1,
                  max = 50,
                  value = 30),
      selectInput("type",label="Select form of distribution",
                  choices=c("normal","uniform")),
      numericInput("number","Number of samples",500,min=100,max=10000,step=100,width='100%'),
      
      
      tags$a("Link to wikipedia",href="http://en.wikipedia.org"),
      tags$p(),
      tags$hr(),
      tags$a( tags$img(src="https://upload.wikimedia.org/wikipedia/en/c/c9/Michigan_Technological_University_logo.svg"),
              href="http://mtu.edu")
    ),
    
    # Show a plot of the generated distribution
    mainPanel(
      h1("Histograms"),
      plotOutput("distPlot"),
      plotOutput("distPlot2",width=400, height=400),
      actionButton("save","Save table to log file"),
      tableOutput("table"),
      h3("Overview in text:"),
      textOutput("textdesc")
    )
  )
)

server <- function(input, output,session) {
  data <- reactive(
    {
      
      if( input$type=="normal")
      {
        x    <-  rnorm(input$number)  
        updateNumericInput(session,"bins",value=25)
      }else if(input$type=="uniform")
      {
        x    <-  runif(input$number) 
        updateNumericInput(session,"bins",value=10)
      }
      
      x
    }
    
      
  )
  observeEvent(input$save,
               {
                 tmp <- summary(data())
                 print(tmp)
                 
                 write.table(t(as.vector(tmp)),file="logfile.csv",append=T,sep=",",
                             col.names=F)
                 print("writing log file")
                 
                 
                 })
  
  
  
  output$distPlot <- renderPlot({
    x <- data()
    bins <- seq(min(x), max(x), length.out = input$bins + 1)
    
    par(mfrow=c(1,1))
    # draw the histogram with the specified number of bins
    hist(x, breaks = bins, col = 'gold', border = 'white')
    
  })
  
  output$distPlot2 <- renderPlot({
    x <- data()
    
    par(mfrow=c(1,1))
    # draw the histogram with the specified number of bins
    qqnorm(x)
    
  })
  
  output$table<- renderTable({
    x <- data()
    summary(data.frame(x))
  })
  
  output$textdesc <- renderText(
    paste("using ",input$type," distribution with a mean of ", round(mean(data()),3),
          " and standard devation of",round(sd(data()),3),"\n")
  )
  
}

# Run the application 
shinyApp(ui = ui, server = server)
