#
# This is a Shiny web application. You can run the application by clicking
# the 'Run App' button above.
#
# Find out more about building applications with Shiny here:
#
#    http://shiny.rstudio.com/
#
## Variable selection
## For this final app, we will add a tab that will plot two of the four variables in the data set USArrests against one another.  
##
## To do this, we'd like to select specific variables, This is kind of like selectInput, but shiny has its own 
## special-purpose version in varSelectInput().
## to do this:
##  *  be sure to use data("USArrests") in the .R file
##  * add a new tab that includes a varSelectInput and a new plotOutput
##  * Add a server function that renders the plot based on the selected plots (see examples in shiny documentation.)

library(plotly)
library(shiny)
library(ggplot2)

data(USArrests)
# Define UI for application that draws a histogram
ui <- fluidPage(
  
  
  # Application title
  titlePanel("Random distribution generation"),
  
  # Sidebar with a slider input for number of bins 
  tabsetPanel(
    tabPanel("Control",
          sliderInput("bins",                        "Number of bins:",
                  min = 1,
                  max = 50,
                  value = 30),
      selectInput("type",label="Select form of distribution",
                  choices=c("normal","uniform")),
      numericInput("number","Number of samples",500,min=100,max=10000,step=100,width='100%'),
      
      
      tags$a("Link to wikipedia",href="http://en.wikipedia.org"),
      tags$p(),
      tags$hr(),
      tags$a( tags$img(src="https://upload.wikimedia.org/wikipedia/en/c/c9/Michigan_Technological_University_logo.svg"),
              href="http://mtu.edu")
    ),
    
    # Show a plot of the generated distribution
          tabPanel("histogram",
             h1("Histograms"),
             plotOutput("distPlot")),
          tabPanel("ggplot",
             plotOutput("distPlot2",width=400, height=400),
             plotOutput("niceplot",width=400, height=400)),
          tabPanel("Plotly",
             plotlyOutput("niceplot2",width=800, height=800)
             , 
      actionButton("save","Save table to log file"),
      tableOutput("table"),
      h3("Overview in text:"),
      textOutput("textdesc")
    ),
     tabPanel("Arrests",
               varSelectInput("vars","Choose variables",USArrests,multiple=FALSE),
               plotOutput("arrestplot")
              )
    
    ))
  


server <- function(input, output,session) {
  data <- reactive(
    {
      
      if( input$type=="normal")
      {
        x    <-  rnorm(input$number)  
        updateNumericInput(session,"bins",value=25)
      }else if(input$type=="uniform")
      {
        x    <-  runif(input$number) 
        updateNumericInput(session,"bins",value=10)
      }
      
      x
    }
    
      
  )
  observeEvent(input$save,
               {
                 tmp <- summary(data())
                 print(tmp)
                 
                 write.table(t(as.vector(tmp)),file="logfile.csv",append=T,sep=",",
                             col.names=F)
                 print("writing log file")
                 
                 
                 })
  
  
  
  output$distPlot <- renderPlot({
    x <- data()
    bins <- seq(min(x), max(x), length.out = input$bins + 1)
    
    par(mfrow=c(1,1))
    # draw the histogram with the specified number of bins
    hist(x, breaks = bins, col = 'gold', border = 'white')
    
  })
  
  output$distPlot2 <- renderPlot({
    x <- data()
    
    par(mfrow=c(1,1))
    # draw the histogram with the specified number of bins
    qqnorm(x)
    
  })
  
  
  output$niceplot <- renderPlot({
    
    x <- data()

    y <-  x + rnorm(length(x)) * 2
    tmp <- data.frame(x,y)
    p <- ggplot(tmp,aes(x=x,y=y)) + geom_point()
    print(p)
  })
  
  
  
  output$niceplot2 <- renderPlotly({
    
    x <- data()
    y <-  x + rnorm(length(x)) * 2
    tmp <- data.frame(x,y)
    plot_ly(tmp,x=x,y=y,type="scatter",mode="markers")

  })
  
  
  output$arrestplot <- renderPlot({
    ggplot(USArrests,aes(!!input$vars)) + geom_histogram()
    
  })
  
  output$table<- renderTable({
    x <- data()
    summary(data.frame(x))
  })
  
  output$textdesc <- renderText(
    paste("using ",input$type," distribution with a mean of ", round(mean(data()),3),
          " and standard devation of",round(sd(data()),3),"\n")
  )
  
}

# Run the application 
shinyApp(ui = ui, server = server)
